use axum::{
    extract::Request,
    middleware::Next,
    response::{IntoResponse, Response},
};

use crate::state::{Router, State};

fn deny() -> Response {
    crate::result::Err::from(eyre::eyre!("nope")).into_response()
}

async fn validate_secret(state: State, request: Request, next: Next) -> Response {
    let Some(val) = request.headers().get("x-secret") else {
        return deny();
    };

    if val != state.secret.as_str() {
        return deny();
    }
    next.run(request).await
}

pub fn make(state: State) -> Router {
    Router::new()
        .nest("/novels", novels::make())
        .layer(axum::middleware::from_fn_with_state(state, validate_secret))
}

mod novels;
