use std::{ops::Deref, sync::Arc};

use crate::{app::App, cfg};

use axum::extract::FromRequestParts;
use parking_lot::RwLock;

pub type Router = axum::Router<State>;

#[derive(Debug)]
pub struct LockedState {
    pub app: RwLock<App>,
    pub cfg: cfg::Root,
    pub secret: String,
}

#[derive(Debug, Clone, FromRequestParts)]
#[from_request(via(axum::extract::State))]
pub struct State(Arc<LockedState>);

impl State {
    pub fn new(app: App, cfg: cfg::Root) -> eyre::Result<Self> {
        Ok(Self(Arc::new(LockedState {
            app: RwLock::new(app),
            secret: cfg.app.secret.into_string()?,
            cfg,
        })))
    }
}

impl Deref for State {
    type Target = LockedState;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}
