use tokio::sync::oneshot;

use std::sync::Arc;

use crate::notifies::Notifies;

#[derive(Debug)]
pub struct Subs {
    queue: Vec<oneshot::Sender<String>>,
    notifies: Arc<Notifies>,
}

impl Subs {
    pub const fn new(notifies: Arc<Notifies>) -> Self {
        Self {
            queue: Vec::new(),
            notifies,
        }
    }

    pub fn has_listeners(&self) -> bool {
        self.queue.len() != 0
    }

    pub fn consume(&mut self) -> Vec<oneshot::Sender<String>> {
        std::mem::take(&mut self.queue)
    }

    pub fn subscribe(&mut self) -> oneshot::Receiver<String> {
        let (tx, rx) = oneshot::channel();
        self.queue.push(tx);
        self.notifies.new_subscriber.notify_waiters();

        rx
    }
}
